/*   ___
   '/ _ \        Version 2.0 - Remote access and
   | | | |  redirection services with strong encryption.
   | |_| |           (c) 1999-2000 by Mixter
   '\__\_\    csa.c - challenge secret authentication
 */

#include "q.h"

unsigned long __id = __ID;

/* Challenge Secret Authentication protocol from the WCML
   (Weak Crypto Mixter Labs)... well, at least it prevents
   having to send the authentication password over the wire
 */

int
csa_put (int outsock, char *p)
{
  int s = strlen (p);
  char *authtok;

  csa_key ();
  authtok = aes_encrypt (p, &s);
  aes_key (p);

  if (write (outsock, authtok, strlen (authtok)) < 1)	/* failed */
    {
      free (authtok);
      return 0;
    }

  free (authtok);
  return 1;			/* key transmitted and set */
}

int
csa_get (int insock)
{
  char authtok[BS], *p, *proof;

  csa_key ();
  memset (authtok, 0, BS);
  if (read (insock, authtok, BS) < 1)
    return -1;

  p = aes_decrypt (authtok, strlen (authtok));
  memset (authtok, 0, BS);
  strncpy (authtok, p, BS);
  aes_key (authtok);

  proof = aes_hash (authtok);
  if (strcmp (proof, __TOK) != 0)	/* authentication failed */
    {
      free (p);
      free (proof);
      return 0;
    }

  free (p);
  free (proof);
  return 1;			/* key verified and set */
}

void
csa_key (void)
{
  char tmp[64], *a;

  memset (tmp, 0, 64);
  snprintf (tmp, 64, "%ld", __id);
  a = aes_hash (tmp);
  strncpy (tmp, a, 64);
  free (a);
  aes_key (tmp);
}
